/***********************************************************************
 *
 *  PROJECT:
 *      ePoch
 *
 *  COPYRIGHT:
 *      Copyright  1999, Sovereign Software, Inc.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public
 *      License along with this program; if not, write to the Free
 *      Software Foundation, Inc., 59 Temple Place, Suite 330,
 *      Boston, MA 02111-1307 USA
 *
 *  FILE:
 *      Epoch.c
 *
 *  DESCRIPTION:
 *      Simple Palm app that calculates the number of days between two
 *      dates. The app requires two of three parameters to perform the
 *      calculation:
 *      (1) Start Date
 *      (2) Num of Days
 *      (3) End Date.
 *
 *      This program was written as a reference on programming style for
 *      Sovereign Software's Partners in Prosperity developers.
 *
 *  TO DO:
 *      Handle prefs
 *
 *  HISTORY:
 *      Sept 10, 1999 - Created by Mark F Rodriguez (macrod@sovware.com)
 *
 **********************************************************************/
#include <Pilot.h>
#include <Graffiti.h>
#include <SysEvtMgr.h>
#include <MemoryPrv.h>

#include "Epoch.h"

/***********************************************************************
 *
 *  FUNCTION:    RomVersionCompatible
 *
 *  DESCRIPTION:
 *      This routine checks that the Palm organizer's ROM meets the
 *      minimum requirements set forth by the application.
 *
 *  PARAMETERS:
 *      requiredVersion     minimum rom version required
 *      launchFlags			indicates if the apps UI is initialized
 *
 *  RETURNED:
 *      error code or zero if ROM is compatible
 *
 *  CALLED BY:
 *      EpochPilotMain
 *
 *  NOTES:
 *      This function is pretty typical in all Palm OS apps.
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static Err
RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
  DWord romVersion;

  // -------------------------------------------------------------------
  // See if device has minimum required version of the ROM.
  // -------------------------------------------------------------------
  FtrGet (sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if (romVersion < requiredVersion)
    {
	  if ((launchFlags &
	        (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
		  FrmAlert (RomIncompatible_Alert);
		
		  // -----------------------------------------------------------
		  // Pilot 1.0 will continuously relaunch this app unless we
		  // switch to another safe one.
		  // -----------------------------------------------------------
		  if (romVersion < sysMakeROMVersion (2, 0, 0,
		        sysROMStageRelease, 0))
            AppLaunchWithCommand (sysFileCDefaultApp,
                                    sysAppLaunchCmdNormalLaunch, NULL);
		}
		
      return (sysErrRomIncompatible);
    }

  return (0);
}

/***********************************************************************
 *
 *  FUNCTION:   GetFocusObjectPtr
 *
 *  DESCRIPTION:
 *      This routine returns a pointer to the field object, in the
 *      current form, that has the focus.
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      pointer to a field object or NULL of there is no fucus
 *
 *  CALLED BY:
 *      MainFormDoCommand
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static FieldPtr
GetFocusObjectPtr (void)
{
  FormPtr           frm;
  Word              focus;
  FormObjectKind    objType;
	
  frm = FrmGetActiveForm ();
  focus = FrmGetFocus (frm);
  if (focus == noFocus)
    return (NULL);
		
  objType = FrmGetObjectType (frm, focus);
	
  if (objType == frmFieldObj)
    return (FrmGetObjectPtr (frm, focus));
	
  else if (objType == frmTableObj)
    return (TblGetCurrentField (FrmGetObjectPtr (frm, focus)));
	
  return (NULL);
}

/***********************************************************************
 *
 *  FUNCTION:   SetFieldTextFromHandle
 *
 *  DESCRIPTION:
 *      Allocates new VoidHand and copies incoming string
 *
 *  PARAMETERS:
 *      fieldID             the filed in question
 *      txtH                text handle
 *
 *  RETURNED:
 *      pointer to a field object or NULL of there is no fucus
 *
 *  CALLED BY:
 *      SetFieldTextFromStr
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static FieldPtr
SetFieldTextFromHandle (Word fieldID, VoidHand txtH)
{
  VoidHand	oldTxtH;
  FormPtr	frm = FrmGetActiveForm();
  FieldPtr	fldP;
	
  // -------------------------------------------------------------------
  // get the field and the field's current text VoidHand
  // -------------------------------------------------------------------
  fldP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, fieldID));
  ErrNonFatalDisplayIf (!fldP, "missing field");
  oldTxtH = (VoidHand) FldGetTextHandle (fldP);
	
  // -------------------------------------------------------------------
  // set the field's text to the new text
  // -------------------------------------------------------------------
  FldSetTextHandle (fldP, (Handle) txtH);
  FldDrawField(fldP);
	
  // -------------------------------------------------------------------
  // free the VoidHand AFTER we call FldSetTextHandle
  // -------------------------------------------------------------------
  if (oldTxtH)
    MemHandleFree (oldTxtH);
	
  return fldP;
}

/***********************************************************************
 *
 *  FUNCTION:   SetFieldTextFromStr
 *
 *  DESCRIPTION:
 *      Allocates new VoidHand and copies incoming string
 *
 *  PARAMETERS:
 *      fieldID             the field in question
 *      strP                pointer to the character string
 *
 *  RETURNED:
 *      pointer to a field object or NULL of there is no fucus
 *
 *  CALLED BY:
 *      EpochDatePicker, DateCalculate
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static FieldPtr
SetFieldTextFromStr (Word fieldID, CharPtr strP)
{
  VoidHand txtH;
	
  // -------------------------------------------------------------------
  // get some space in which to stash the string
  // -------------------------------------------------------------------
  txtH = MemHandleNew (StrLen (strP) + 1);
  if (!txtH)
    return NULL;
		
  // -------------------------------------------------------------------
  // copy the string to the locked VoidHand
  // -------------------------------------------------------------------
  StrCopy (MemHandleLock (txtH), strP);
	
  // -------------------------------------------------------------------
  // unlock the string VoidHand
  // -------------------------------------------------------------------
  MemHandleUnlock (txtH);
	
  // -------------------------------------------------------------------
  // set the field to the VoidHand
  // -------------------------------------------------------------------
  return SetFieldTextFromHandle (fieldID, txtH);
}

/***********************************************************************
 *
 *  FUNCTION:    GetIntFromField
 *
 *  DESCRIPTION:
 *      extract an integer from a field
 *
 *  PARAMETERS:
 *      fldPtr              pointer to the field in question
 *
 *  RETURNED:
 *      pointer to a field object or NULL of there is no fucus
 *
 *  CALLED BY:
 *      DateCalculate
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
unsigned long
GetIntFromField (FieldPtr fldPtr)
{
  Handle          fldHandle;
  CharPtr         data;
  unsigned long   n;

  // -------------------------------------------------------------------
  // get a handle on the field
  // -------------------------------------------------------------------
  fldHandle = (Handle) FldGetTextHandle(fldPtr);
  
  // -------------------------------------------------------------------
  // lock the handle and get a pointer to the character string
  // -------------------------------------------------------------------
  data = (CharPtr) MemHandleLock((VoidHand)fldHandle);
  
  // -------------------------------------------------------------------
  // convert the character string to int; the GUI for this field is set
  // to only accept numerical data
  // -------------------------------------------------------------------
  n = StrAToI(data);
  
  // -------------------------------------------------------------------
  // unlock the handle
  // -------------------------------------------------------------------
  MemHandleUnlock((VoidHand)fldHandle);
  
  // -------------------------------------------------------------------
  // return the integer
  // -------------------------------------------------------------------
  return n;
}

/***********************************************************************
 *
 *  FUNCTION:   SetIntToField
 *
 *  DESCRIPTION:
 *      Set integer value in a Field
 *
 *  PARAMETERS:
 *      fldPtr              a pointer to the field
 *      n                   the integer to set
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      DateCalculate
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
void
SetIntToField (FieldPtr fldPtr, unsigned long n)
{
  Handle    fldHandle;
  CharPtr   data;

  // -------------------------------------------------------------------
  // free current field memory, if any
  // -------------------------------------------------------------------
  FldFreeMemory (fldPtr);
  
  // -------------------------------------------------------------------
  // allocate memory handle and lock it to get pointer to char string
  // -------------------------------------------------------------------
  fldHandle = (Handle) MemHandleNew (80);
  data = (CharPtr) MemHandleLock ((VoidHand) fldHandle);
  
  // -------------------------------------------------------------------
  // convert the string and unlock the handle
  // -------------------------------------------------------------------
  StrIToA (data, n);
  MemHandleUnlock ((VoidHand) fldHandle);
  
  // -------------------------------------------------------------------
  // set the field text to the new memory handle
  // -------------------------------------------------------------------
  FldSetTextHandle (fldPtr, fldHandle);
  
  // -------------------------------------------------------------------
  // redraw the field because the contents have changed
  // -------------------------------------------------------------------
  FldDrawField(fldPtr);
}

/***********************************************************************
 *
 *  FUNCTION:   GetCurrentDate
 *
 *  DESCRIPTION:
 *      This routine gests today's date.
 *
 *  PARAMETERS:
 *      none
 *
 *  RETURNED:
 *      DateType as defined in 'DateTime.h'
 *
 *  CALLED BY:
 *      EpochDatePicker
 *
 *  NOTES:
 *      The value for the year (i.e., today.year) must be increased by
 *      1904 to get the actual year.
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static DateType
GetCurrentDate (void)
{
  DateType today;
 	
  // -------------------------------------------------------------------
  // Get today's date.
  // -------------------------------------------------------------------
  DateSecondsToDate (TimGetSeconds (), &today);
	
  return (today);
}

/***********************************************************************
 *
 *  FUNCTION:   EpochLoadPrefs
 *
 *  DESCRIPTION:
 *      Read the preferences.
 *
 *  PARAMETERS:
 *      nothing.
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *
 *  NOTES:
 *      This function needs to be "worked into" the ePoch applcication.
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
/*
void
EpochLoadPrefs (void)
{
  ePochPreferenceType       prefs;
  Word                      prefsSize;
  Int                       prefsVersion;

  // -------------------------------------------------------------------
  // Read the preferences / saved-state information.
  // -------------------------------------------------------------------
  prefsSize = sizeof (EpochPreferenceType);
  prefsVersion = PrefGetAppPreferences (sysFileCEpoch, epochPrefID,
                                            &prefs, &prefsSize, true);
  if (prefsVersion > epochPrefsVersionNum)
    {
	  prefsVersion = noPreferenceFound;
	}
	
  if (prefsVersion > noPreferenceFound)
    {
	  if (prefsVersion < epochPrefsVersionNum)
	    {
          // add code here
		}
		
      // ---------------------------------------------------------------
      // Set global variables state from the stored preferences.
      // ---------------------------------------------------------------
      PresetDate = prefs.presetDate;
      RemeberPrevDate = prefs.rememberPrevDate;
      PrevEpoch = prefs.prevEpoch;
    }
}*/

/***********************************************************************
 *
 *  FUNCTION:   EpochDatePicker
 *
 *  DESCRIPTION:
 *      This routine presents the Select Date GUI and stores the
 *      user-selected date in the global variable Epoch.
 *
 *  PARAMETERS:
 *      dateFormat          format to display the selected date
 *      isStartDate         set true if user selected the start date
 *
 * RETURNED:
 *      Nothing
 *
 *  CALLED BY:
 *      MainFormHandleEvent
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 14, 1999	MFR		Created
 *
 **********************************************************************/
static void
EpochDatePicker (DateFormatType dateFormat, Boolean isStartDate)
{
  SWord         month, day, year;
  CharPtr       label;
  CharPtr       title;
  Word          correctField;
  DateType      todaysDate;
	
  // -------------------------------------------------------------------
  // Use today's date to initialize 'select date' GUI element
  // -------------------------------------------------------------------
  todaysDate = GetCurrentDate ();
  year = todaysDate.year + firstYear;		// date taken from 1/1/1904
  month = todaysDate.month;
  day = todaysDate.day;
	
  title = MemHandleLock(DmGetResource (strRsc, DateTitle_String));

  if (SelectDay (selectDayByDay, &month, &day, &year, title))
    {
      // ---------------------------------------------------------------
      // Store and put info in the appropriate place
      // ---------------------------------------------------------------
      if (isStartDate)
        {
          Epoch.startDate.day = day;
		  Epoch.startDate.month = month;
		  Epoch.startDate.year = year - firstYear;
		  Epoch.hasStartDate = true;
		  correctField = Main_StartDate_Field;
		}
      else
		{
		  Epoch.endDate.day = day;
		  Epoch.endDate.month = month;
		  Epoch.endDate.year = year - firstYear;
		  Epoch.hasEndDate = true;
		  correctField = Main_EndDate_Field;
		}
		
      DateToDOWDMFormat (month, day, year, dateFormat, label);
      FldDrawField (SetFieldTextFromStr (correctField, label));
    }

  MemPtrUnlock (title);
}

/***********************************************************************
 *
 *  FUNCTION:   ClearEpochData
 *
 *  DESCRIPTION:
 *      This routine reinitializes the epoch data structure and clears
 *      the main form
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      MainFormHandleEvent
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 14, 1999	MFR		Created
 *
 **********************************************************************/
 static void
 ClearEpochData (void)
 {
  FormPtr	    frm = FrmGetActiveForm();
  FieldPtr	    fldP;
  Word          end;
	
  // -------------------------------------------------------------------
  // make sure user has to enter in the appropriate info before doing
  // another calc
  // -------------------------------------------------------------------
  Epoch.hasStartDate = false;
  Epoch.hasEndDate = false;
  Epoch.hasNumOfDays = false;
	
  // -------------------------------------------------------------------
  // get the start date field and erase it!
  // -------------------------------------------------------------------
  fldP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,
                            Main_StartDate_Field));
  ErrNonFatalDisplayIf (!fldP, "missing field");
  FldEraseField (fldP);
	
  // -------------------------------------------------------------------
  // do the same for the number of days field
  // -------------------------------------------------------------------
  fldP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,
                            Main_NumOfDays_Field));
  ErrNonFatalDisplayIf (!fldP, "missing field");
  end = FldGetTextLength (fldP);
  FldDelete (fldP, 0, end);
	
  // -------------------------------------------------------------------
  // get the end date field and erase it too!
  // -------------------------------------------------------------------
  fldP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,
                            Main_EndDate_Field));
  ErrNonFatalDisplayIf (!fldP, "missing field");
  FldEraseField (fldP);
 }

/***********************************************************************
 *
 *  FUNCTION:   DateCalculate
 *
 *  DESCRIPTION:
 *      Calculates the number of days between two dates or the date that
 *		is a specified number of days from another date. This routine
 *		handles three different scenerios:
 *			endDate = startDate + numOfDays
 *			startDate = endDate - numOfDays
 *			numOfDays = endDate - startDate
 *
 *		This routine also checks the date to make sure it falls between
 *		1/1/1904 and 12/31/2031. If it doesn't, an alert dialog
 *      is displayed. If the user doesn't enter at least two parameters
 *      no calculations are performed and a value of false is returned.
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      true if one of the three scenerios handled or false if there was
 *		not enough info or the date was not inbetween the Palm OS limits
 *
 *  CALLED BY:
 *      MainFormHandleEvent
 *
 *  NOTES:
 *      Set up a preference structure to allow the user to select
 *      whether they prefer the ShortDateFormat (i.e., 9/12/99) or
 *      LongDateFormat (i.e., Spet 12, 1999). It would be nice to see a
 *      Date class defined in C++ so that these functions could be
 *      reused.
 *
 *  HISTORY:
 *      Sept 14, 1999	MFR		Created
 *
 **********************************************************************/
static Boolean
EpochDateCalculate (void)
{
  FormPtr			frmP;
  FieldPtr			fldPtr;
  FieldAttrType		attr;
  Long				numOfDays;
  Long				daysInBetween;
  Long				daysTillOrigin;
  Long				daysTillDoom;
  ULong				startDateInDays;
  ULong				endDateInDays;
  ULong				calculatedDateInDays;
  DateType			calculatedDate;
  CharPtr			label;
  Boolean			handled;

  // -------------------------------------------------------------------
  // Get the attribute of the # of Days field to check if it's been
  // changed.
  // -------------------------------------------------------------------
  frmP = FrmGetActiveForm();
  fldPtr = FrmGetObjectPtr (frmP, FrmGetObjectIndex (frmP,
                            Main_NumOfDays_Field));
  FldGetAttributes (fldPtr, &attr);
  
  // -------------------------------------------------------------------
  // Based on the values entered by the user, initialize the variables
  // to perform the correct calculation.
  // -------------------------------------------------------------------
  if (Epoch.hasStartDate)
	{
	  startDateInDays = DateToDays (Epoch.startDate);
	}
 	
  if (Epoch.hasEndDate)
	{
	  endDateInDays = DateToDays (Epoch.endDate);
	}
 	
  if (attr.dirty)
	{
	  numOfDays = GetIntFromField (fldPtr);
	  Epoch.numOfDays = numOfDays;
	  Epoch.hasNumOfDays = true;
	}
 	
  // -------------------------------------------------------------------
  // Handle the appropriate date calculation based on the available data
  // entered by the user.
  // -------------------------------------------------------------------
  if (Epoch.hasStartDate && Epoch.hasEndDate)
	{
	  daysInBetween = endDateInDays - startDateInDays;
 	  SetIntToField (fldPtr, daysInBetween);
 	  handled = true;
 	}
 	
  else if (Epoch.hasStartDate && Epoch.hasNumOfDays)
	{
	  // ---------------------------------------------------------------
	  // make sure number of days advanced falls within
	  // 1/1/1904 - 12/31/2031
	  // ---------------------------------------------------------------
	  daysTillDoom = maxDays - startDateInDays;
	  if (Epoch.numOfDays > daysTillDoom)
		FrmAlert(OutOfBounds_Alert);
	  else
		{
		  calculatedDateInDays = startDateInDays + numOfDays;
		  DateDaysToDate (calculatedDateInDays, &calculatedDate);
		  Epoch.endDate = calculatedDate;
		  DateToDOWDMFormat (Epoch.endDate.month, Epoch.endDate.day,
		      Epoch.endDate.year + firstYear, ShortDateFormat, label);
		  FldDrawField (SetFieldTextFromStr (Main_EndDate_Field, label));
		}
	  handled = true;
	}
 	
  else if (Epoch.hasEndDate && Epoch.hasNumOfDays)
	{
	  // ---------------------------------------------------------------
	  // make sure number of days advanced falls within
	  // 1/1/1904 - 12/31/2031
	  // ---------------------------------------------------------------
	  daysTillOrigin = endDateInDays;
	  if (Epoch.numOfDays > daysTillOrigin)
		FrmAlert(OutOfBounds_Alert);
	  else
		{
		  calculatedDateInDays = endDateInDays - numOfDays;
		  DateDaysToDate (calculatedDateInDays, &calculatedDate);
		  Epoch.startDate = calculatedDate;
		  DateToDOWDMFormat (Epoch.startDate.month, Epoch.startDate.day,
		  	  Epoch.startDate.year + firstYear, ShortDateFormat, label);
		  FldDrawField (SetFieldTextFromStr (Main_StartDate_Field, label));
		}
	  handled = true;
	}
 	
  // -------------------------------------------------------------------
  // There was not enough info to perform any of the above calculations
  // -------------------------------------------------------------------
  else
	handled = false;
 	
  return (handled);
}

/***********************************************************************
 *
 *  FUNCTION:   AboutDisplay
 *
 *  DESCRIPTION:
 *      This routine displays the ePoch About Form.
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      MainFormDoCommand
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
 static void
 AboutDisplay (void)
 {
   FormPtr  curFormP;
   FormPtr  formP;
 	
   // ------------------------------------------------------------------
   // Find the current active form and save it for later restoration
   // ------------------------------------------------------------------
   curFormP = FrmGetActiveForm ();
 	
   // ------------------------------------------------------------------
   // Load and initialize the about box form
   // ------------------------------------------------------------------
   formP = FrmInitForm (About_Form);
   FrmSetActiveForm (formP);
 	
   // ------------------------------------------------------------------
   // Run the about box as a dialog until the 'ok' button is pressed
   // ------------------------------------------------------------------
   FrmDoDialog (formP);
 	
   // ------------------------------------------------------------------
   // Delete the about box form and restore the Main form
   // ------------------------------------------------------------------
   FrmDeleteForm (formP);
   FrmSetActiveForm (curFormP);
 }
 
/***********************************************************************
 *
 *  FUNCTION:   MainFormInit
 *
 *  DESCRIPTION:
 *      This routine initializes the MainForm with data saved in the
 *      apps preferences.
 *
 *  PARAMETERS:
 *      frm         pointer to the MainForm form.
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      MainFormHandleEvent
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static void
MainFormInit (FormPtr frmP)
{
  // -------------------------------------------------------------------
  // add code to pull info like the previous date calculation from the
  // saved preferences
  // -------------------------------------------------------------------
}

/***********************************************************************
 *
 *  FUNCTION:   MainFormDoCommand
 *
 *  DESCRIPTION:
 *      This routine performs the menu command specified.
 *
 *  PARAMETERS:
 *      command         menu item id
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      MainFormHandleEvent
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static Boolean
MainFormDoCommand (Word command)
{
  FieldPtr 	    fld;
  Boolean		handled = true;

  if (command == Options_AboutePoch)
    {
      AboutDisplay();
	}
	
  // -------------------------------------------------------------------
  // Undo the last text change
  // -------------------------------------------------------------------
  else if (command == Edit_Undo)
    {
      fld = GetFocusObjectPtr ();
      if (fld)
        FldUndo (fld);
	}
	
  // -------------------------------------------------------------------
  // Do a cut to the clipboard if text is highlighted
  // -------------------------------------------------------------------
  else if (command == Edit_Cut)
    {
	  fld = GetFocusObjectPtr ();
	  if (fld)
        FldCut (fld);
	}
	
  // -------------------------------------------------------------------
  // Copy the highlighted text to the clipboard
  // -------------------------------------------------------------------
  else if (command == Edit_Copy)
    {
      fld = GetFocusObjectPtr ();
      if (fld)
        FldCopy (fld);
	}
	
  // -------------------------------------------------------------------
  // Do a paste from the clipboard
  // -------------------------------------------------------------------
  else if (command == Edit_Paste)
    {
	  fld = GetFocusObjectPtr ();
	  if (fld)
        FldPaste (fld);
	}
	
  // -------------------------------------------------------------------
  // Select all the existing text
  // -------------------------------------------------------------------
  else if (command == Edit_SelectAll)
    {
	  fld = GetFocusObjectPtr ();
	  if (fld)
		FldSetSelection (fld, 0, FldGetTextLength (fld));
	}
	
  // -------------------------------------------------------------------
  // Display the on screen keyboard
  // -------------------------------------------------------------------
  else if (command == Edit_Keyboard)
    SysKeyboardDialog (kbdNumbersAndPunc);
	
  // -------------------------------------------------------------------
  // Display the graffiti reference screen
  // -------------------------------------------------------------------
  else if (command == Edit_GraffitiHelp)
    SysGraffitiReferenceDialog (referenceDefault);
	
  // -------------------------------------------------------------------
  // Catch-all; something went wrong
  // -------------------------------------------------------------------
  else
    handled = false;
			

  return handled;
}

/***********************************************************************
 *
 *  FUNCTION:   MainFormHandleEvent
 *
 *  DESCRIPTION:
 *      This routine is the event handler for the "MainForm" of this
 *      application.
 *
 *  PARAMETERS:
 *      eventP      a pointer to an EventType structure
 *
 *  RETURNED:
 *      true if the event has handle and should not be passed to a
 *      higher level handler.
 *
 *  CALLED BY:
 *      ApplicationHandleEvent
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static Boolean
MainFormHandleEvent (EventPtr eventP)
{
  FormPtr   frmP;
  FieldPtr  fldPtr;
  Word      fldId;
  
  Boolean handled = false;

  switch (eventP->eType) 
    {
      case menuEvent:
        return MainFormDoCommand (eventP->data.menu.itemID);

	  case frmOpenEvent:
		frmP = FrmGetActiveForm ();
		MainFormInit (frmP);
		FrmDrawForm (frmP);
		fldId = FrmGetObjectIndex (frmP, Main_NumOfDays_Field);
		fldPtr = (FieldPtr) FrmGetObjectPtr (frmP, fldId);
		FrmSetFocus (frmP, fldId);
		handled = true;
		break;
		
	  case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID)
		  {
			case Main_StartDate_Button:
			  EpochDatePicker (ShortDateFormat, true);
			  handled = true;
			  break;
				
			case Main_EndDate_Button:
			  EpochDatePicker (ShortDateFormat, false);
			  handled = true;
			  break;
				
			case Main_Clear_Button:
			  ClearEpochData ();
			  handled = true;
			  break;
				
			case Main_Calculate_Button:
			  if (!EpochDateCalculate ())
			    FrmAlert(NotEnoughData_Alert);
			  handled = true;
			  break;
          }
        break;
			
      default:
        break;
		
    }
	
  return handled;
}

/***********************************************************************
 *
 *  FUNCTION:   ApplicationHandleEvent
 *
 *  DESCRIPTION:
 *      This routine loads form resources and sets the event handler for
 *      the form loaded.
 *
 *  PARAMETERS:
 *      event       a pointer to an EventType structure
 *
 *  RETURNED:
 *      true if the event has a handle and should not be passed to a
 *      higher level handler.
 *
 *  CALLED BY:
 *      EventLoop
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static Boolean
ApplicationHandleEvent (EventPtr eventP)
{
  Word      formId;
  FormPtr   frmP;

  if (eventP->eType == frmLoadEvent)
    {
      // ---------------------------------------------------------------
      // Load the form resource.
      // ---------------------------------------------------------------
	  formId = eventP->data.frmLoad.formID;
	  frmP = FrmInitForm(formId);
	  FrmSetActiveForm (frmP);

	  // ---------------------------------------------------------------
	  // Set the event handler for the form.  The handler of the
	  // currently active form is called by FrmHandleEvent each time it
	  // receives an event.
	  // ---------------------------------------------------------------
	  if (formId == Main_Form)
	    FrmSetEventHandler(frmP, MainFormHandleEvent);
		
	  /*else if (formId == Prefs_Form)
		FrmSetEventHandler(frmP, PrefsFormHandleEvent);*/
		
	  return (true);
	}
  return (false);
}

/***********************************************************************
 *
 *  FUNCTION:   EventLoop
 *
 *  DESCRIPTION:
 *      This routine is the event loop for the application.  
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      EpochPilotMain
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static void
EventLoop (void)
{
  Word          error;
  EventType     event;

  do
    {
      EvtGetEvent (&event, evtWaitForever);
		
      if (! SysHandleEvent (&event))
        if (! MenuHandleEvent (0, &event, &error))
          if (! ApplicationHandleEvent (&event))
            FrmDispatchEvent (&event);
	}
  while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 *  FUNCTION:   StartApplication
 *
 *  DESCRIPTION:
 *      Get the system prefs for the date format along with the current
 *      application's preferences. We also initialze the Epoch global
 *      variable.
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      Err value 0 if nothing went wrong
 *
 *  CALLED BY:
 *      EpochPilotMain
 *
 *  NOTES:
 *      Eventually we should handle the applications preferences.
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static Err
StartApplication (void)
{
  EpochPreferenceType       prefs;
  SystemPreferencesType     sysPrefs;
  SWord                     prefsVersion;
  Word                      prefsSize;
    
  // -------------------------------------------------------------------
  // Get the date formats from the system preferences
  // -------------------------------------------------------------------
  PrefGetPreferences (&sysPrefs);
  //LongDateFormat = sysPrefs.longDateFormat;
  ShortDateFormat = sysPrefs.dateFormat;
    
  // -------------------------------------------------------------------
  // Get today's date
  // -------------------------------------------------------------------
  //Epoch.todaysDate = GetCurrentDate();
    
  // -------------------------------------------------------------------
  // Read the saved preferences / saved-state information.
  // -------------------------------------------------------------------
  prefsSize = sizeof (EpochPreferenceType);
  if (PrefGetAppPreferences (appFileCreator, appPrefID, &prefs,
        &prefsSize, true) != noPreferenceFound)
    {
      // add code here to handle the apps preferences
    }
	
  // -------------------------------------------------------------------
  // Clear the ePoch data structure
  // -------------------------------------------------------------------
  MemSet (&Epoch, sizeof(Epoch), 0);
	
  // -------------------------------------------------------------------
  // Some initial values
  // -------------------------------------------------------------------
  Epoch.hasStartDate = false;
  Epoch.hasEndDate = false;
  Epoch.hasNumOfDays = false;
  Epoch.numOfDays = 0;
	
  return 0;
}

/***********************************************************************
 *
 *  FUNCTION:   StopApplication
 *
 *  DESCRIPTION:
 *      Save the current state of the application.
 *
 *  PARAMETERS:
 *      nothing
 *
 *  RETURNED:
 *      nothing
 *
 *  CALLED BY:
 *      EpochPilotMain
 *
 *  NOTES:
 *      Need to implement data structure and functions to handle the
 *      apps preferences. Currently, this code really does nothing
 *      besides close all (i.e., the main) forms.
 *
 *  HISTORY:
 *      Sept 12, 1999	MFR		Created
 *
 **********************************************************************/
static void
StopApplication (void)
{
  EpochPreferenceType prefs;
   
  // -------------------------------------------------------------------
  // Write the saved preferences / saved-state information.  This data 
  // will be backed up during a HotSync.
  // -------------------------------------------------------------------
  PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
	
  // -------------------------------------------------------------------
  // Close all open forms
  // -------------------------------------------------------------------
  FrmCloseAllForms();
}

/***********************************************************************
 *
 *  FUNCTION:   EpochPilotMain
 *
 *  DESCRIPTION:
 *      This routine handles different launch codes and sets the
 *      interface to the 'main' form. It is here where all the functions
 *      for the main entry point into the ePoch app can be found (i.e.,
 *      StartApplication, EventLoop, and StopApplication).
 *  
 *  PARAMETERS:
 *      cmd             word value specifying the launch code. 
 *      cmdPB           pointer to a structure that is associated with
 *                      the launch code. 
 *      launchFlags     word value providing extra information about
 *                      the launch.
 *
 *  RETURNED:
 *      Result of launch
 *
 *  CALLED BY:
 *      PilotMain
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 10, 1999	MFR		Created
 *
 **********************************************************************/
static DWord
EpochPilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  Err error;

  // -------------------------------------------------------------------
  // make sure we are version compliant
  // -------------------------------------------------------------------
  error = RomVersionCompatible (version20, launchFlags);
  if (error)
    return (error);
	
  // -------------------------------------------------------------------
  // Launch code sent by the launcher or one of the hard buttons
  // -------------------------------------------------------------------
  if (cmd == sysAppLaunchCmdNormalLaunch)
    {
      error = StartApplication();
      if (error)
        return (error);
		
      FrmGotoForm(Main_Form);
      EventLoop();
      StopApplication();
    }
	
  // -------------------------------------------------------------------
  // Respond to country change (i.e., 10/19/99 vs 19/10/99)
  // -------------------------------------------------------------------
  else if (cmd == sysAppLaunchCmdCountryChange)
    {
      // add code to handle this case
	}
	
  // -------------------------------------------------------------------
  // Allow user to change date formats in the system prefs panel from
  // within the ePoch application.
  // -------------------------------------------------------------------
  else if (cmd == sysAppLaunchCmdPanelCalledFromApp)
    {
      // add code to handle this case
	}
	
  // -------------------------------------------------------------------
  // Restart the app following changes to the prefs panel
  // -------------------------------------------------------------------
  else if (cmd == sysAppLaunchCmdReturnFromPanel)
	{
	  // add code to handle this case
	}
	
  // -------------------------------------------------------------------
  // Respond to a time change initiated by the user
  // -------------------------------------------------------------------
  else if (cmd == sysAppLaunchCmdTimeChange)
	{
	  // add code to handle this case
	}
	
  return error;
}

/***********************************************************************
 *
 *  FUNCTION:      PilotMain
 *
 *  DESCRIPTION:
 *      This is the main entry point for the ePoch application.
 *
 *  PARAMETERS:
 *      cmd             word value specifying the launch code. 
 *      cmdPB           pointer to a structure that is associated with
 *                      the launch code. 
 *      launchFlags     word value providing extra information about
 *                      the launch.
 *
 *  RETURNED:
 *      Result of launch
 *
 *  CALLED BY:
 *      Nothing. This is where all the action begins.
 *
 *  NOTES:
 *
 *  HISTORY:
 *      Sept 10, 1999	MFR		Created
 *
 **********************************************************************/
DWord
PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  return EpochPilotMain(cmd, cmdPBP, launchFlags);
}
